function cvresult = doublecv(Data, varargin)

% Runs a double cross-validation to evaluate the performance of the method
% whose name is specified in the method.name field of Data.
% In each fold of the outer CV, some positive examples are left unlabeled
% then these folds are subdivided into folds for inner CV, the method is
% trained and tested, the best parameters out of 'param' are selected and
% used to predict scores on the outer test set for all negatives.
% If no argument param is specified, a simple cross validation is
% performed to evaluate the performance of parameters specified in
% Data.method.param
% If an argument NbPos is given, an 'inversed' CV will be carried out to
% evaluate the performance of the method when used with NbPos positive
% examples. The positive set is divided into as many sets of NbPos as
% possible and each subset is used in turn as training set for outer CV.
% If an additionnal argument Nrepeats is present, the function will
% run a 'bootstrap' cross-validation : the positive set will be sampled to
% form Nrepeats sets of NbPos examples.
%
% USE :
% cvresult = doublecv(Data)
% or cvresult = doublecv(Data, struct('C', [1 10 100]))
% or cvresult = doublecv(Data, struct('C', [1 10 100]), 'NbPos', 10)
% or cvresult = doublecv(Data, struct('C', [1 10 100]), 'NbPos', 10, 'Nrepeats', 20)
%
% INPUT :
%  - Data : a structure containing the features, the positive examples and
%  various parameters, as initialized by initialize()
%
% OPTIONAL ARGUMENTS :
%  - NbPos : an integer indicating how many positive examples should be used
%  for learning.
%  - Nrepeats : an integer which makes the function perform a bootstrap
%  validation. Nrepeats sets of NbPos positive examples are randomly
%  chosen for learning.
%  - param : a structure containing the different parameters among which to
%  select. It must contain at least 'C', a grid for the SVM regularization
%  parameter. Then, for bagging, the structure can contain 'K', a vector of
%  values for the size of the subsample used as negative train set and
%  optionally, 'r', a grid for the unbalance ratio between positive and
%  negative examples (r=C+/C-)

%
% OUTPUT :
%  - an array of structures, each containing:
%       o score : a matrix of predicted scores
%         or a structure as built by one of the 'predict' functions,
%         containing amongst other information, the predicted scores, using
%         the best parameters, selected in the inner loop of CV
%       o label : a list of +1/0/-1 labels corresponding to the scores
%       o bestParam : the selected parameters


NbPos=NaN;
Nrepeats=NaN;

% Parse optional arguments';
l=1;

while (l<length(varargin))
    eval([varargin{l},' = varargin{',num2str(l+1),'};'])
    l = l+2;
end

posSplit = split(Data,'Nrepeats', Nrepeats, 'NbPos', NbPos);

% preallocating
cvresult = cell(1, length(posSplit));
% initialization
for k=1:length(posSplit)
    cvresult{k} = struct('label', [], 'score', [], 'param', []);
end
sData = Data;

for ifold = 1:length(posSplit)
    disp(['######### Now starting iteration ',int2str(ifold),' out of ',int2str(length(posSplit))])
    
    % run method with only the training examples of the fold as positive
    
    % assign labels: -1 for true negative, 0 for true positive used as
    % positive training examples, 1 for true positive used as negative
    % training examples
    
    if (~isnan(NbPos)) && isnan(Nrepeats)
        % 'inversed CV'
        sData.posSet = posSplit{ifold}.test;
        
        label = -ones(1,Data.nexample);
        label(posSplit{ifold}.test) = 0;
        label(posSplit{ifold}.train) = 1;
    else
        % bootstrap validation or classical CV
        sData.posSet = posSplit{ifold}.train;
        
        label = -ones(1,Data.nexample);
        label(posSplit{ifold}.train) = 0;
        label(posSplit{ifold}.test) = 1;
    end
    cvresult{ifold}.label = label';
    
    % compute scores for the fold
    if ~exist('param','var')
        % simple CV for one parameter
        res=predict(sData);
        switch lower(sData.method.name)
            case {'baggingsvm','baggingmkl'}
                cvresult{ifold}.score = res.score;
                cvresult{ifold}.scoreBrut = res.scoreBrut;
            otherwise
                cvresult{ifold}.score = res;
        end
    else
        % double CV with internal loop for parameter selection and record 
        % the results of the outer fold
        if ~isfield(param, 'r')
            param.r=0;
        end
        if ~isfield(param, 'K')
            param.K=NaN;
        end
        sData.nsplitCvOut = Data.nsplitCvInt;
        switch lower(sData.method.name)
            case {'baggingsvm','baggingmkl'}
                res = cvSelect(sData, param);
                cvresult{ifold}.param = [res.bestParam.C', res.bestParam.K', res.bestParam.r'];
                cvresult{ifold}.score = res.score.score;
                cvresult{ifold}.scoreBrut = res.score.scoreBrut;
            otherwise
                res = cvSelect(sData, param);
                cvresult{ifold}.param = res.bestParam;
                cvresult{ifold}.score = res.score;
        end
    end
end

